local addon, db = ...

db.init_reagents = function ()
	if db.SV.DONT_BUY_REAGENTS then return end

	local reagents = {}

	local check_glyphs = false
	local glyph_is_missing
	local update_reagents

	local class = select(2, UnitClass"player")
	if class == "DRUID" then
		check_glyphs = true -- Glyph of Unburdened Rebirth
		update_reagents = function (level)
			-- Rebirth
			if glyph_is_missing(57857) then
				if level >= 24 then
					reagents[17034] = 1
				end
			end
		end
	elseif class == "MAGE" then
		update_reagents = function (level)
			-- Rune of Teleportation
			if level >= 24 then
				reagents[17031] = 1
			end
			-- Rune of Portals
			if level >= 42 then
				reagents[17032] = 1
			end
		end
	elseif class == "SHAMAN" then
		check_glyphs = true -- Glyph of Renewed Life
		update_reagents = function (level)
			-- Ankh
			if level >= 30 and glyph_is_missing(58059) then
				reagents[17030] = 1
			end
		end
	elseif class == "ROGUE" then
		update_reagents = function (level)
			-- Instant Poison
			if level >= 10 then
				reagents[6947] = 1
			end
			-- Crippling Poison
			if level >= 20 then
				reagents[3775] = 1
			end
			-- Mind-numbing Poison
			if level >= 24 then
				reagents[5237] = 1
			end
			-- Deadly Poison
			if level >= 30 then
				reagents[2892] = 1
			end
			-- Wound Poison
			if level >= 32 then
				reagents[10918] = 1
			end
		end
	else
		update_reagents = nil
	end

	if not update_reagents then return end

	db.Register(function (self)
		for item, stacks in pairs(reagents) do
			local link = select(2, GetItemInfo(item))
			if link then
				local maxStack = select(8, GetItemInfo(item))
				local count = stacks * maxStack
				local have = GetItemCount(link)
				local tobuy = count - have
				if tobuy > 0 then
					for i = 1, GetMerchantNumItems() do
						if GetMerchantItemLink(i) == link then
							local quantity = select(4, GetMerchantItemInfo(i))
							while tobuy > 0 do
								local c = math.min(tobuy, maxStack)
								c = math.floor(c / quantity)
								if c == 0 then break end
								BuyMerchantItem(i, c)
								tobuy = tobuy - (c * quantity)
							end
							break
						end
					end
				end
			end
		end
		return true
	end, "MERCHANT_SHOW")

	db.Register(function (self)
		update_reagents(UnitLevel"player")
	end, "PLAYER_ENTERING_WORLD")

	db.Register(function (self, _, level)
		reagents = {}
		update_reagents(tonumber(level))
		return true
	end, "PLAYER_LEVEL_UP")

	if not check_glyphs then return end

	local function has_glyph(spellId, talentGroup)
		for socket = 1, NUM_GLYPH_SLOTS do
			local enabled, _, spell = GetGlyphSocketInfo(socket, talentGroup)
			if spell == spellId then return true end
		end
	end

	if db.SV.GLYPH_ONCE_IS_ENOUGH then
		glyph_is_missing = function (spellId)
			for talentGroup = 1, GetNumTalentGroups() do
				if has_glyph(spellId, talentGroup) then
					return false
				end
			end
			return true
		end
	else
		glyph_is_missing = function (spellId)
			for talentGroup = 1, GetNumTalentGroups() do
				if not has_glyph(spellId, talentGroup) then
					return true
				end
			end
		end
	end

	db.Register(function (self)
		reagents = {}
		update_reagents(UnitLevel"player")
		return true
	end, "GLYPH_ADDED", "GLYPH_REMOVED", "GLYPH_UPDATED")
end
